/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.db.connectionfactory.ConfigurationException;
import com.topcoder.db.connectionfactory.DBConnectionException;
import com.topcoder.db.connectionfactory.DBConnectionFactory;
import com.topcoder.db.connectionfactory.DBConnectionFactoryImpl;
import com.topcoder.db.connectionfactory.UnknownConnectionException;
import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.persistence.impl.Utils;

import java.lang.reflect.Constructor;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Arrays;


/**
 * This is an abstraction of a typical data base driven persistence contract. This provides for getting a connection
 * from the DBConnectionFactory as well as specifying transaction control. This hides nicely the intricacies of
 * configuration that would be used by all derived classes.
 * <p/>
 * <b>Thread safety:</b>This class is no designed to be thread-safe.
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public abstract class AbstractDBIndexPersistence implements IndexPersistence {

    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} lookup key used for reading the connection factory
     * class name from the ConfigManager.
     */
    private static final String CONNECTION_FACTORY_CLASS_NAME_PROPERTY_NAME = "ConnectionFactoryClassName";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} lookup key used for reading the connection factory
     * namespace from the ConfigManager.
     */
    private static final String CONNECTION_FACTORY_NAMESPACE_PROPERTY_NAME = "ConnectionFactoryNamespace";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} lookup key used for reading the {@link
     * #connectionName} from the ConfigManager.
     */
    private static final String CONNECTION_NAME_PROPERTY_NAME = "ConnectionName";

    /**
     * This field holds the connection factory from which we will obtain a pre-configured connection for our data base
     * access. This is initialized in one of the constructors. Cannot be <tt>null</tt>.
     */
    private final DBConnectionFactory connectionFactory;

    /**
     * This field holds a connection name (an alias) that is used to fetch a connection instance from the connection
     * factory. This is initialized in one of the constructors and once initialized cannot be changed. Can be
     * <tt>null</tt> or an empty string, upon which it will try to use the default connection.
     */
    private final String connectionName;

    /**
     * This constructor will populate the connectionFactory and connectionName information from configuration.
     *
     * @param namespace The namespace for connection-related properties.
     *
     * @throws IllegalArgumentException if namespace param is <tt>null</tt> or an empty (trim'd) String
     * @throws PersistenceConfigurationException
     *                                  if there are configuration issues encountered
     */
    protected AbstractDBIndexPersistence(final String namespace) throws PersistenceConfigurationException {
        if (namespace == null) {
            throw new IllegalArgumentException("The parameter named [namespace] was null.");
        }
        if (namespace.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [namespace] was an empty String.");
        }

        final String connectionFactoryClass = Utils.lookupOptionalStringFromConfigManager(namespace,
            CONNECTION_FACTORY_CLASS_NAME_PROPERTY_NAME);
        final String connectionFactoryNamespace = Utils.lookupValidStringFromConfigManager(namespace,
            CONNECTION_FACTORY_NAMESPACE_PROPERTY_NAME);
        if (connectionFactoryClass == null) {
            try {
                connectionFactory = new DBConnectionFactoryImpl(connectionFactoryNamespace);
            } catch (ConfigurationException e) {
                throw new PersistenceConfigurationException(
                    "The DBConnectionFactory encountered a misconfiguration issue.",
                    e);
            } catch (UnknownConnectionException e) {
                throw new PersistenceConfigurationException(
                    "The DBConnectionFactory encountered a misconfiguration issue.",
                    e);
            }
        } else {
            connectionFactory = (DBConnectionFactory) createInstance(connectionFactoryClass,
                new Class[]{String.class}, new Object[]{connectionFactoryNamespace}, DBConnectionFactory.class);
        }
        connectionName = Utils.lookupOptionalStringFromConfigManager(namespace, CONNECTION_NAME_PROPERTY_NAME);
    }

    /**
     * This constructor will populate the connectionFactory and connectionName information from input parameters.
     *
     * @param connectionFactory DBConnectionFactory instance used to generate connections.
     * @param connectionName    The name of the connection used for persistence operations.
     *
     * @throws IllegalArgumentException if connectionFactory is <tt>null</tt>
     */
    protected AbstractDBIndexPersistence(final DBConnectionFactory connectionFactory, final String connectionName) {
        if (connectionFactory == null) {
            throw new IllegalArgumentException("The parameter named [connectionFactory] was null.");
        }
        this.connectionFactory = connectionFactory;
        // simplify empty String to null
        this.connectionName = connectionName != null
                              ? connectionName.trim().length() > 0
                                ? connectionName
                                : null
                              : null;
    }

    /**
     * Returns a new connection to be used for persistence. This method will obtain the connection from the
     * connectionFactory using the connectionName alias.
     *
     * @return the transactional connection created
     *
     * @throws IndexPersistenceException If unable to obtain a Connection.
     */
    protected Connection getConnection() throws IndexPersistenceException {
        try {
            final Connection connection = connectionName == null
                                          ? connectionFactory.createConnection()
                                          : connectionFactory.createConnection(connectionName);
            connection.setAutoCommit(false);
            return connection;
        } catch (DBConnectionException e) {
            throw new IndexPersistenceException(
                "Connecting to the database (connection name was [" + connectionName + "]) failed.", e);
        } catch (SQLException e) {
            throw new IndexPersistenceException("Initialization of connection parameters (autocommit) "
                + "failed on the connection obtained from DBConnectionFactory (connection name was ["
                + connectionName + "]).", e);
        }
    }

    /**
     * Starts a transaction on the passed in connection.
     *
     * @param connection The Connection taking part in the transaction.
     *
     * @throws IndexPersistenceException If an error occurs during this process, or if the Connection does not support
     *                                   transactions.
     * @throws IllegalArgumentException  if connection is <tt>null</tt>
     */
    protected void beginTransaction(final Connection connection) throws IndexPersistenceException {
        if (connection == null) {
            throw new IllegalArgumentException("The parameter named [connection] was null.");
        }

        // as stated by the designer this method currently does nothing
    }

    /**
     * This method commits a transaction on the passed in connection. The connection is the closed.
     *
     * @param connection The Connection taking part in the transaction to be committed and closed
     *
     * @throws IndexPersistenceException If an error occurs during this process, or if the Connection does not support
     *                                   transactions
     * @throws IllegalArgumentException  if connection is <tt>null</tt>
     */
    protected void commitTransaction(final Connection connection) throws IndexPersistenceException {
        if (connection == null) {
            throw new IllegalArgumentException("The parameter named [connection] was null.");
        }

        try {
            connection.commit();
        } catch (SQLException e) {
            throw new IndexPersistenceException("Error while committing the connection [" + connection + "]", e);
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method rolls back the transaction on  the passed connection.
     *
     * @param connection The Connection taking part in the transaction to be rolled back and closed
     *
     * @throws IndexPersistenceException If an error occurs during this process, or if the Connection does not support
     *                                   transactions
     * @throws IllegalArgumentException  if connection is <tt>null</tt>
     */
    protected void rollbackTransaction(final Connection connection) throws IndexPersistenceException {
        if (connection == null) {
            throw new IllegalArgumentException("The parameter named [connection] was null.");
        }

        try {
            connection.rollback();
        } catch (SQLException e) {
            throw new IndexPersistenceException("Error while rolling back the connection [" + connection + "]", e);
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method creates a new Object instance from the given class name. If the class could be loaded, a Constructor
     * that has the signature specified is looked up, which is then called specifying the given initargs.
     *
     * @param classname            the name of the implementation class to be instantiated
     * @param constructorSignature the signature of the constructor to be called
     * @param initargs             the constructor arguments to be used upon invocation of the constructor retrieved
     * @param expectedType         the expected type of the instance created, will be checked after instantiation
     *
     * @return the instance created
     *
     * @throws PersistenceConfigurationException
     *                                  the class specified cannot be loaded,no public constructor with the signature
     *                                  specified does exist in the class or the constructor invocation throws an
     *                                  exception
     * @throws IllegalArgumentException if any arg is <tt>null</tt>, or any String arg is an empty (trim'd) string or
     *                                  the init args do not match the given signature
     */
    private static Object createInstance(final String classname,
                                         final Class[] constructorSignature,
                                         final Object[] initargs,
                                         final Class expectedType) throws PersistenceConfigurationException {
        //arg checking is done in findConstructor
        final Constructor constructor = findConstructor(classname, initargs, expectedType, constructorSignature);

        //create instance
        final Object instance;
        try {
            instance = constructor.newInstance(initargs);
        } catch (Throwable t) {
            // catch of Throwable is normally discouraged by the TC coding style, but
            // as a lot of Errors can arise from a reflection invocation and these
            // errors don't need to be let pass through this catch clause (as they
            // definitely belong to the reflective invocation and not the normal program
            // execution) all Throwables are caught here and
            // wrapped into a ConfigurationException.
            throw new PersistenceConfigurationException("the implementation class [" + classname
                + "]  could not be instantiated, constructor invocation failed.", t);
        }
        return instance;
    }

    /**
     * This method loads the class from the given class name. If the class could be loaded, a Constructor that has the
     * signature specified is looked up.
     *
     * @param classname            the name of the implementation class to be loaded
     * @param constructorSignature the signature of the constructor to be fond
     * @param initargs             the constructor arguments to be used upon invocation of the constructor retrieved
     *                             (for checking conformance with given signature)
     * @param expectedType         the expected type of the class loaded, if not assignable, ConfigurationException will
     *                             be thrown
     *
     * @return the Constructor found
     *
     * @throws PersistenceConfigurationException
     *                                  the class specified cannot be loaded,no public constructor with the signature
     *                                  specified does exist in the class or the constructor invocation throws an
     *                                  exception
     * @throws IllegalArgumentException if any arg is <tt>null</tt>, or any String arg is an empty (trim'd) string or
     *                                  the init args do not match the given signature or the constructor signature
     *                                  contains a <tt>null</tt> value
     */
    private static Constructor findConstructor(final String classname, final Object[] initargs,
                                               final Class expectedType,
                                               final Class[] constructorSignature)
        throws PersistenceConfigurationException {
        if (classname == null) {
            throw new IllegalArgumentException("The parameter named [classname] was null.");
        }
        if (classname.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [classname] was an empty String.");
        }
        if (initargs == null) {
            throw new IllegalArgumentException("The parameter named [initargs] was null.");
        }
        if (expectedType == null) {
            throw new IllegalArgumentException("The parameter named [expectedType] was null.");
        }
        if (constructorSignature == null) {
            throw new IllegalArgumentException("The parameter named [constructorSignature] was null.");
        }
        checkArgsMatchSignature(constructorSignature, initargs);

        //lookup and load instance implementation class
        final Class implClass;
        try {
            implClass = Class.forName(classname);
        } catch (Throwable t1) {
            // catch of Throwable is normally discouraged by the TC coding style, but
            // as a lot of Errors can arise from a class load and these
            // errors don't need to be let pass through this catch clause (as they
            // definitely belong to the class load and not the normal program
            // execution) all Throwables are caught here and
            // wrapped into a ConfigurationException.
            throw new PersistenceConfigurationException("Unable to load implementation class [" + classname + "].", t1);
        }

        //check if class matches expected type
        if (!expectedType.isAssignableFrom(implClass)) {
            throw new PersistenceConfigurationException(
                "the implementation class [" + classname + "]  is not of type [" + expectedType.getName() + "].");
        }

        //find constructor
        final Constructor constructor;
        try {
            constructor = implClass.getConstructor(constructorSignature);
        } catch (NoSuchMethodException e) {
            throw new PersistenceConfigurationException("The implementation class [" + implClass .getName()
                + "]  did not have a constructor with signature [" + Arrays.asList(constructorSignature).toString()
                + "].", e);

        }
        return constructor;
    }

    /**
     * This method checks whether each element of the initargs is assignable to the type specified in the equivalent
     * constructorSignature element, both arrays have same size and constructorSignature does not contain <tt>null</tt>
     * values.
     *
     * @param constructorSignature the signature to check for match with initargs
     * @param initargs             the initargs to check for match with constructorSignature
     *
     * @throws IllegalArgumentException in case any of the checks fails and thus the given initargs cannot be used as
     *                                  args for a constructor of the given signature
     */
    private static void checkArgsMatchSignature(final Class[] constructorSignature, final Object[] initargs) {
        if (initargs.length != constructorSignature.length) {
            throw new IllegalArgumentException("The given constructor signature ([" + constructorSignature.length
                + "] args) and the given initargs ([" + initargs.length + "] args) have different length.");
        }
        for (int i = 0; i < constructorSignature.length; i++) {
            final Class argType = constructorSignature[i];
            final Object argValue = initargs[i];
            if (argType == null) {
                throw new IllegalArgumentException(
                    "The parameter named [constructorSignature] contained a null value.");
            }

            // a null argument does match any type
            if (argValue != null) {
                // check if actual arg matches declared arg type of constructor signature
                if (!argType.isAssignableFrom(argValue.getClass())) {
                    throw new IllegalArgumentException("The given initArg element at position [" + i + "] is of type ["
                        + argValue.getClass().getName()
                        + "] which does not match the argument type specified in the constructor signature ["
                        + argType.getName() + "] for that argument.");
                }
            }
        }
    }

}
