/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.IndexPersistenceException;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.persistence.impl.Utils;
import com.topcoder.document.index.wordsource.WordSourceId;
import com.topcoder.util.compression.Base64Codec;
import com.topcoder.util.compression.Base64Decoder;
import com.topcoder.util.compression.Base64Encoder;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.CollationKey;
import java.text.Collator;
import java.text.MessageFormat;
import java.text.StringCharacterIterator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.zip.DataFormatException;


/**
 * This is a concrete implementation of the IndexPersistence contract that uses XML files as storage medium.
 * <p/>
 * This implementation allows for creation of arbitrarily large document index files by sequencing (splitting) large
 * files into chunks of file based data. Since it is required that binary data is stored in the XML files for document
 * index a serialized Base64 encoded data will be both encoded and decoded by this implementation.
 * <p/>
 * <b>Thread safety:</b>This class is no designed to be thread-safe.
 *
 * @author AleaActaEst, TCSDEVELOPER
 * @version 1.0
 */
public class XmlIndexPersistence implements IndexPersistence {

    /**
     * This constant is the file name used to represent a document usage as defined by the persistence contract. There
     * is only one such file per XML persistence name space.
     */
    private static final String DOC_USAGE_FILE_NAME = "document_usage.xml";

    /**
     * This is the file name prefix for all files that represent a Collection Index XML file storage (i.e. a document
     * collection file) Each collection XML file name will have the following format:
     * <tt>doc_collection_collectionId.xml</tt>.
     * <p/>
     * For example here is a simple document collection XML file for a collection with id of <tt>175658</tt>:
     * <tt>doc_collection_175658.xml</tt>.
     */
    private static final String DOC_COLLECTION_FILE_NAME_PREFIX = "doc_collection_";

    /**
     * This is the file name prefix for all files that represent a Document Index XML file storage (i.e. a document
     * index file) Each document index file name will have the following format:<tt>doc_index_docId_sequenceid.xml</tt>.
     * <p/>
     * Here is an example of a document index XML file for a document with id of <tt>16527</tt>:
     * <tt>doc_index_16527_001.xml</tt>.
     * <p/>
     * The <tt>sequenceid</tt> is used to link files with data that is larger than a configured limit of e.g. 2GB
     * (this can be configured) this means that if there is indexed data that is larger than say 2GB the file is split
     * into 2 files with the first having the sequence id of 001 and the second one being obviously 002. This gives the
     * flexibility for persistence of very large content.
     */
    private static final String DOC_INDEX_FILE_NAME_PREFIX = "doc_index_";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} lookup key used for reading the {@link
     * #persistenceDirPath} from the ConfigManager.
     */
    private static final String XML_PERSISTENCE_PATH_PROPERTY_NAME = "XmlPersistencePath";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} lookup key used for reading the {@link
     * #fileSizeThreshold} from the ConfigManager.
     */
    private static final String FILE_SIZE_THRESHOLD_PROPERTY_NAME = "FileSizeThreshold";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} lookup key used for reading the {@link #fileSizeLimit}
     * from the ConfigManager.
     */
    private static final String FILE_SIZE_LIMIT_PROPERTY_NAME = "FileSizeLimit";
    /**
     * This constant defines the charset used in XML encoding.
     */
    private static final String UTF_8_CHARSET = "UTF-8";
    /**
     * This constant defines the factor used in converting kilobyte values into byte-values and megabyte values into
     * kilobytes.
     */
    private static final int KILO_FACTOR = 1024;

    /**
     * This is a file size limit for saving a chunk of indexed data. It is measured in Mbytes. Consider data so large
     * that it cannot fit into a single file due to size limitations for the specific OS or file system. This size limit
     * is used to decide when to start chaining multiple files to store the data. It is initialized from configuration
     * through the constructor and once initialized cannot be changed. Must be a positive integer (i.e.
     * <tt>&gt;0</tt>).
     */
    private final int fileSizeLimit;

    /**
     * This is a safety net size in Kbytes that is applied before the  fileSizeLimit is reached. This is basically
     * something of a round off error precaution. It is initialized from configuration through the constructor and once
     * initialized cannot be changed. Must be a non-negative integer (i.e. <tt>&gt;= 0</tt>).
     */
    private final int fileSizeThreshold;

    /**
     * This is the file path to a directory where all the XML files that make up this persistence session are stored (it
     * is almost like a data base namespace). This must be a valid fully qualified path. It is initialized from
     * configuration through the constructor and once initialized cannot be changed. Cannot be <tt>null</tt> or an empty
     * string.
     */
    private final String persistenceDirPath;

    /**
     * This is a simple constructor which will populate the fileSizeLimit, fileSizeThreshold, and persistenceDirPath
     * information from configuration by looking up the values from the given {@link
     * com.topcoder.util.config.ConfigManager} namespace.
     *
     * @param namespace the configuration namespace to be used for looking up the values from the {@link
     *                  com.topcoder.util.config.ConfigManager}
     *
     * @throws PersistenceConfigurationException
     *                                  if there are configuration issues encountered, e.g. missing or invalid values
     * @throws IllegalArgumentException if namespace param is <tt>null</tt> or an empty (trim'd) String
     */
    public XmlIndexPersistence(final String namespace) throws PersistenceConfigurationException {
        if (namespace == null) {
            throw new IllegalArgumentException("The parameter named [namespace] was null.");
        }
        if (namespace.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [namespace] was an empty String.");
        }
        fileSizeLimit = Utils.lookupIntFromConfigManager(namespace, FILE_SIZE_LIMIT_PROPERTY_NAME, 1) * KILO_FACTOR
            * KILO_FACTOR;
        fileSizeThreshold = Utils.lookupIntFromConfigManager(namespace, FILE_SIZE_THRESHOLD_PROPERTY_NAME, 0)
            * KILO_FACTOR;
        try {
            persistenceDirPath = checkPath(
                Utils.lookupValidStringFromConfigManager(namespace, XML_PERSISTENCE_PATH_PROPERTY_NAME));
        } catch (IllegalArgumentException e) {
            throw new PersistenceConfigurationException("The configured persistence directory was not usable.", e);
        }
        //check whether the directory contains a valid document-usages file
        try {
            readDocumentUsages();
        } catch (IndexPersistenceException e) {
            throw new PersistenceConfigurationException("The persistence directory configured [" + persistenceDirPath
                + "] did not contain a valid document_usages.xml.", e);
        }
    }

    /**
     * This is a simple convenience constructor which will populate the fileSizeLimit, fileSizeThreshold, and
     * persistenceDirPath information directly. The fileSizeLimit and fileSizeThreshold values should be in a meaningful
     * relation to each other, otherwise it could happen that every word is persisted to a separate file (when
     * <tt>fileSizeLimit * 1024 &lt;= fileSizeThreshold</tt> ).
     *
     * @param fileSizeLimit      This is a file size limit in Mbytes for saving a chunk of indexed data, must be
     *                           <tt>&gt;0</tt>
     * @param fileSizeThreshold  This is a safety net size in Kbytes that is applied before the  fileSizeLimit is
     *                           reached, must be <tt>&gt;=0</tt>
     * @param persistenceDirPath This is the file path to a directory where all the XML files that make up this
     *                           persistence session are stored
     *
     * @throws IllegalArgumentException in case fileSizeLimit is &lt;1 or fileSizeThreshold is &lt;0 or the given
     *                                  persistenceDirPath does not denote an existent absolute path of a directory that
     *                                  is writable and contains a valid document_usage.xml file
     */
    public XmlIndexPersistence(final int fileSizeLimit, final int fileSizeThreshold, final String persistenceDirPath) {
        if (fileSizeLimit < 1) {
            throw new IllegalArgumentException(
                "The parameter named [fileSizeLimit] was expected to be >0 , but was [" + fileSizeLimit + "].");
        }
        if (fileSizeThreshold < 0) {
            throw new IllegalArgumentException(
                "The parameter named [fileSizeThreshold] was expected to be >=0 , but was [" + fileSizeThreshold
                    + "].");
        }
        this.fileSizeLimit = fileSizeLimit * KILO_FACTOR * KILO_FACTOR;
        this.fileSizeThreshold = fileSizeThreshold * KILO_FACTOR;
        this.persistenceDirPath = checkPath(persistenceDirPath);
        //check whether the directory contains a valid document-usages file
        try {
            readDocumentUsages();
        } catch (IndexPersistenceException e) {
            final IllegalArgumentException illegalArgumentException = new IllegalArgumentException(
                "The persistence directory configured [" + persistenceDirPath
                    + "] did not contain a valid document_usages.xml.");
            illegalArgumentException.initCause(e);
            throw illegalArgumentException;
        }
    }

    /**
     * This method adds the given DocumentIndex to the persistence.
     *
     * @param documentIndex WordSource representing the document to be indexed, should not be <tt>null</tt>
     *
     * @throws IllegalArgumentException  when documentIndex is <tt>null</tt>
     * @throws IndexPersistenceException when the method fails to add DocumentIndex to the persistence
     */
    public void addDocumentIndex(final DocumentIndex documentIndex) throws IndexPersistenceException {
        if (documentIndex == null) {
            throw new IllegalArgumentException("The parameter named [documentIndex] was null.");
        }

        final String documentID = Utils.createIdString(documentIndex.getWordSourceId());

        final File firstFile = new File(persistenceDirPath, DOC_INDEX_FILE_NAME_PREFIX + documentID + "_"
            + 1 + ".xml");
        if (firstFile.exists()) {
            throw new IndexPersistenceException("The given index does already exist in the persistent storage as file ["
                + firstFile.getName() + "].");
        }

        final Locale locale = documentIndex.getLocale();
        final WordSourceId sourceId = documentIndex.getWordSourceId();

        // serialize the source identity and encode it into a Base64 String
        final byte[] input = Utils.serializeObject(sourceId.getSourceIdentity());
        final byte[] bytes = new byte[input.length * 2];
        final Base64Encoder base64Encoder = new Base64Encoder(0, null, Base64Codec.STANDARD_ALPHABET);
        base64Encoder.setInput(input);
        base64Encoder.finish();
        final int size = base64Encoder.deflate(bytes);
        final String sourceIdentityString = new String(bytes, 0, size);

        //build the delimiters list
        final String[] delimiters = sourceId.getDelimiters();
        final StringBuffer temp = new StringBuffer();
        for (int i = 0; i < delimiters.length; i++) {
            temp.append("<entry delimiter=\"");
            temp.append(escapeForXML(delimiters[i]));
            temp.append("\"/>");
        }
        final String delimiterEntries = temp.toString();

        // this has not been extracted to be a string constant
        // as the readability and maintainability would be
        // decreased when separating the format message with
        // its placeholders from the actual values used
        // for the placeholders
        final String header = MessageFormat.format(
            "<?xml version=\"1.0\" encoding=\"" + UTF_8_CHARSET + "\" standalone=\"yes\"?>"
                + "<doc lang=\"{0}-{1}\">"
                + "<document-index document-id=\"{2}\">"
                + "<word-source-data>"
                + "<source-identity>{3}</source-identity>"
                + "<class-name>{4}</class-name>"
                + "<locale language=\"{0}\" country=\"{1}\" variant=\"{5}\"/>"
                + "<delimiters>{6}</delimiters>"
                + "</word-source-data>",
            new Object[]{
                escapeForXML(locale.getLanguage()),
                escapeForXML(locale.getCountry()),
                escapeForXML(documentID),
                sourceIdentityString, // unescaped as Base64 is ASCII only
                escapeForXML(sourceId.getSourceClassName()),
                escapeForXML(locale.getVariant()),
                delimiterEntries // unescaped as already escaped during construction
            }
        );

        final String footer = "</document-index></doc>";

        writeDocumentDataFiles(header, footer, documentID, documentIndex);

        //insert the entry representing the document into document_usages.xml
        final Map map = readDocumentUsages();
        map.put(documentID, new DocumentUsage(documentID));
        writeDocumentUsages(map);
    }

    /**
     * This method retrieves a DocumentIndex with the specified WordSourceId from the persistence. It does return
     * <tt>null</tt>, if DocumentIndex with the given WordSourceId is not found in the persistence.
     * <p/>
     * CS section 1.4.4.1 describes the algorithm of this method.
     *
     * @param wordSourceId WordSourceId of document to retrieve
     *
     * @return the retrieved DocumentIndex, or <tt>null</tt> if document index with given WordSourceId is not found in
     *         the persistence
     *
     * @throws IllegalArgumentException  when wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when the implementation fails to retrieve document index with the specified
     *                                   WordSourceId
     */
    public DocumentIndex getDocumentIndex(final WordSourceId wordSourceId) throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);

        return getDocumentIndex(documentId);
    }

    /**
     * This method removes a document index with given WordSourceId from the persistence. When the document index is not
     * found, or function fail to remove it, an exception is thrown. Function does also not succeed when document index
     * use count is not zero.
     * <p/>
     * CS section 1.4.4.2  describes the algorithm of this method.
     *
     * @param wordSourceId WordSourceId of  document index to remove
     *
     * @throws IllegalArgumentException  when WordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when document index is not found in the persistence, its use count is not zero,
     *                                   or error happens when trying to remove it
     */
    public void removeDocumentIndex(final WordSourceId wordSourceId) throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final int count = getDocumentUseCount(wordSourceId);
        if (count != 0) {
            throw new IndexPersistenceException("The index with the given id [" + wordSourceId
                + "] cannot be deleted as its use count is not 0, but is " + count + ".");
        }

        final String documentId = Utils.createIdString(wordSourceId);
        final File[] indexFiles = new File(persistenceDirPath).listFiles(
            new PrefixSuffixFilter(DOC_INDEX_FILE_NAME_PREFIX + documentId, ".xml"));
        for (int i = 0; i < indexFiles.length; i++) {
            final File indexFile = indexFiles[i];
            indexFile.delete();
        }

        // update document_usages.xml
        // (remove element representing document)
        final Map map = readDocumentUsages();
        map.remove(documentId);
        writeDocumentUsages(map);
    }

    /**
     * This method stores the given document collection index in the persistence.
     * <p/>
     * CS section 1.4.4.4  describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to store
     *
     * @throws IllegalArgumentException  if collectionIndex is <tt>null</tt>
     * @throws IndexPersistenceException if fails to create and store document collection index or the given index
     *                                   already exists in persistence or contains unpersisted documents
     */
    public void addCollectionIndex(final CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        if (collectionIndex == null) {
            throw new IllegalArgumentException("The parameter named [collectionIndex] was null.");
        }

        final String collectionId = collectionIndex.getId();
        final File persistenceFile =
            new File(persistenceDirPath, DOC_COLLECTION_FILE_NAME_PREFIX + collectionId + "_" + ".xml");

        if (persistenceFile.exists()) {
            throw new IndexPersistenceException("The given Collection index does already exist in persistence as file ["
                + persistenceFile.getName() + "].");
        }

        //build the documents list
        final Set allDocumentIds = collectionIndex.getAllDocumentIds();
        // this is a set of the string form of the document ids,
        // re-used when updating the usage information
        final Set documentIdStrings = new HashSet();
        final StringBuffer temp = new StringBuffer();
        for (Iterator iterator = allDocumentIds.iterator(); iterator.hasNext();) {
            final WordSourceId sourceId = (WordSourceId) iterator.next();
            final String documentId = Utils.createIdString(sourceId);

            //check whether document has been persisted
            final String expectedFileName =
                DOC_INDEX_FILE_NAME_PREFIX + documentId + "_" + 1 + ".xml";
            if (!new File(persistenceDirPath, expectedFileName).exists()) {
                throw new IndexPersistenceException(
                    "Cannot add collection when its contained documents are not yet persisted (document with id ["
                        + sourceId + "] was not persisted).");
            }

            documentIdStrings.add(documentId);
            temp.append("<document-id>");
            temp.append(escapeForXML(documentId));
            temp.append("</document-id>");
        }
        final String documentList = temp.toString();

        // this has not been extracted to be a string constant
        // as the readability and maintainability would be
        // decreased when separating the format message with
        // its placeholders from the actual values used
        // for the placeholders
        final String fileContent = MessageFormat.format(
            "<?xml version=\"1.0\" encoding=\"" + UTF_8_CHARSET + "\" standalone=\"yes\"?>"
                + "<document-collection collection-id=\"{0}\">"
                + "<documents>{1}</documents>"
                + "</document-collection>",
            new Object[]{
                escapeForXML(collectionId),
                documentList  // already escaped
            }
        );

        //write out the data to the file
        writeUTF8File(fileContent, persistenceFile);

        if (!documentIdStrings.isEmpty()) {
            //update document usages
            final Map documentUsages = readDocumentUsages();
            for (Iterator iterator = documentIdStrings.iterator(); iterator.hasNext();) {
                final String documentId = (String) iterator.next();
                DocumentUsage usage = (DocumentUsage) documentUsages.get(documentId);
                if (usage == null) {
                    usage = new DocumentUsage(documentId);
                    documentUsages.put(documentId, usage);
                }
                usage.add(collectionId);
            }
            writeDocumentUsages(documentUsages);
        }
    }

    /**
     * This method retrieves a document collection index with specified identifier. Id does return <tt>null</tt> when
     * collection with specified identifier is not found
     * <p/>
     * CS section 1.4.4.5  describes the algorithm of this method.
     *
     * @param collectionId identifier of document collection index to retrieve
     *
     * @return CollectionIndex with specified identifier or <tt>null</tt> if the collection index does not exist in
     *         persistence
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or empty string
     * @throws IndexPersistenceException when collection index can not be retrieved
     */
    public CollectionIndex getCollectionIndex(final String collectionId) throws IndexPersistenceException {
        if (collectionId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }
        if (collectionId.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [collectionId] was an empty String.");
        }

        final File persistenceFile =
            new File(persistenceDirPath, DOC_COLLECTION_FILE_NAME_PREFIX + collectionId + "_" + ".xml");

        if (!persistenceFile.exists()) {
            return null;
        }

        final DocumentCollectionHandler dh = new DocumentCollectionHandler();
        saxParse(persistenceFile, dh);
        final String[] documentIds = dh.getDocumentIds();

        final Map wordsOfCollection = new HashMap();
        final Set documentSourceIds = new HashSet();

        for (int i = 0; i < documentIds.length; i++) {
            final String documentId = documentIds[i];
            final DocumentIndex index = getDocumentIndex(documentId);
            final WordSourceId sourceId = index.getWordSourceId();

            // add all words of the index to the collection's word-index
            // map and add a reference to the current index
            final Set wordsOfIndex = index.getWords().keySet();
            for (Iterator iterator = wordsOfIndex.iterator(); iterator.hasNext();) {
                final CollationKey collationKey = (CollationKey) iterator.next();
                Set documentsCotainingWord = (Set) wordsOfCollection.get(collationKey);
                if (documentsCotainingWord == null) {
                    documentsCotainingWord = new HashSet();
                    wordsOfCollection.put(collationKey, documentsCotainingWord);
                }
                documentsCotainingWord.add(sourceId);
            }
            // add document to collection's contained documents set
            documentSourceIds.add(sourceId);
        }
        // build the collection object and return it
        return new CollectionIndex(null, wordsOfCollection, documentSourceIds, collectionId);
    }

    /**
     * This method does remove document collection index with specified identifier from the persistence.
     * <p/>
     * CS section 1.4.4.7 describes the algorithm of this method.
     *
     * @param collectionId identifier of document  collection index to remove
     *
     * @throws IllegalArgumentException  when collectionId is <tt>null</tt> or empty string
     * @throws IndexPersistenceException when fails to remove CollectionIndex with given identifier or the collection
     *                                   does not exist in persistence
     */
    public void removeCollectionIndex(final String collectionId) throws IndexPersistenceException {
        if (collectionId == null) {
            throw new IllegalArgumentException("The parameter named [collectionId] was null.");
        }
        if (collectionId.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [collectionId] was an empty String.");
        }
        final File persistenceFile =
            new File(persistenceDirPath, DOC_COLLECTION_FILE_NAME_PREFIX + collectionId + "_" + ".xml");

        if (!persistenceFile.exists()) {
            throw new IndexPersistenceException(
                "The Collection index with the given id [" + collectionId + "] does not exist in persistence as file ["
                    + persistenceFile.getName() + "].");
        }
        //delete the index file
        persistenceFile.delete();

        //update document usages
        final Map documentUsages = readDocumentUsages();

        final Collection usages = documentUsages.values();
        for (Iterator iterator = usages.iterator(); iterator.hasNext();) {
            final DocumentUsage documentUsage = (DocumentUsage) iterator.next();
            documentUsage.remove(collectionId);
        }

        writeDocumentUsages(documentUsages);
    }

    /**
     * This method does update specified CollectionIndex in the persistence.
     * <p/>
     * CS section 1.4.4.6  describes the algorithm of this method.
     *
     * @param collectionIndex CollectionIndex to update
     *
     * @throws IllegalArgumentException  if collectionIndex parameter is <tt>null</tt>
     * @throws IndexPersistenceException if any error happens when updating the collection index in the persistence,
     *                                   this includes the case, when the specified collectionIndex is not found in the
     *                                   persistence
     */
    public void updateCollectionIndex(final CollectionIndex collectionIndex)
        throws IndexPersistenceException {
        if (collectionIndex == null) {
            throw new IllegalArgumentException("The parameter named [collectionIndex] was null.");
        }
        // There is no real overhead in implementing the method this
        // way and it avoids a lot of code duplication too
        removeCollectionIndex(collectionIndex.getId());
        addCollectionIndex(collectionIndex);
    }

    /**
     * This method increases the use count value for document index with specified WordSourceId.
     *
     * @param wordSourceId WordSourceId of document index which to update use count
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to increase document index use count in the persistence or document
     *                                   index with id specified does not exist in persistence
     */
    public void increaseDocumentUseCount(final WordSourceId wordSourceId)
        throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);

        final Map documentUsages = readDocumentUsages();
        final DocumentUsage usage = (DocumentUsage) documentUsages.get(documentId);
        if (usage == null) {
            throw new IndexPersistenceException("The index document with the given word source id [" + wordSourceId
                + "], string value is [" + documentId + "] does not exist in the persistence.");
        }
        usage.incUsageCount();

        writeDocumentUsages(documentUsages);
    }

    /**
     * This method decreases the use count value for document index with specified WordSourceId.
     *
     * @param wordSourceId WordSourceId of document index of which to update use count
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to decrease document index use count in the persistence or document
     *                                   index with id specified does not exist in persistence
     */
    public void decreaseDocumentUseCount(final WordSourceId wordSourceId)
        throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);

        final Map documentUsages = readDocumentUsages();
        final DocumentUsage usage = (DocumentUsage) documentUsages.get(documentId);
        if (usage == null) {
            throw new IndexPersistenceException("The index document with the given word source id [" + wordSourceId
                + "], string value is [" + documentId + "] does not exist in the persistence.");
        }
        usage.decUsageCount();

        writeDocumentUsages(documentUsages);
    }

    /**
     * This method does return set of WordSourceId of documents that have been persisted in this persistence instance.
     * <p/>
     * CS section 1.4.4.1  describes the algorithm of this method.
     *
     * @return set of WordSourceId of documents that have been persisted
     *
     * @throws IndexPersistenceException when fails to retrieve the ids
     */
    public Set getIndexedDocuments() throws IndexPersistenceException {
        final Set ret = new HashSet();
        final File[] indexFiles = new File(persistenceDirPath).listFiles(
            new PrefixSuffixFilter(DOC_INDEX_FILE_NAME_PREFIX, "_1.xml"));

        if (indexFiles == null) {
            throw new IndexPersistenceException("An error has occurred while scanning the persistence directory ["
                + persistenceDirPath + "] for matching persistence files.");
        }

        for (int i = 0; i < indexFiles.length; i++) {
            final WordSourceIdHandler dh = new WordSourceIdHandler();
            saxParse(indexFiles[i], dh);
            ret.add(dh.createWordSourceId());
        }
        return ret;
    }

    /**
     * This method does return the count of how many different groups this document is a member of.
     *
     * @param wordSourceId WordSourceId of document index of which to retrieve use count
     *
     * @return the use count of the document with the given id
     *
     * @throws IllegalArgumentException  if wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when fails to retrieve document index use count in the persistence or document
     *                                   index with id specified does not exist in persistence
     */
    public int getDocumentUseCount(final WordSourceId wordSourceId)
        throws IndexPersistenceException {
        if (wordSourceId == null) {
            throw new IllegalArgumentException("The parameter named [wordSourceId] was null.");
        }

        final String documentId = Utils.createIdString(wordSourceId);

        final Map documentUsages = readDocumentUsages();
        final DocumentUsage usage = (DocumentUsage) documentUsages.get(documentId);
        if (usage == null) {
            throw new IndexPersistenceException("The index document with the given word source id [" + wordSourceId
                + "], string value is [" + documentId + "] does not exist in the persistence.");
        }
        return usage.getUsageCount();
    }

    /**
     * This method checks whether the given non-<tt>null</tt> non-empty String denotes a Path to a existent and writable
     * directory.
     *
     * @param path the path to be checked
     *
     * @return the checked path
     *
     * @throws IllegalArgumentException in case the given path violates any of the rules stated above
     */
    private String checkPath(final String path) {
        if (path == null) {
            throw new IllegalArgumentException("The parameter named [path] was null.");
        }
        if (path.trim().length() == 0) {
            throw new IllegalArgumentException("The parameter named [path] was an empty String.");
        }

        final File file = new File(path);
        if (!file.exists()) {
            throw new IllegalArgumentException("The given path [" + path + "] does not exist.");
        }
        if (!file.isDirectory()) {
            throw new IllegalArgumentException("The given path [" + path + "] was not a directory.");
        }
        if (!file.canRead()) {
            throw new IllegalArgumentException("The given path [" + path + "] is not readable.");
        }
        if (!file.canWrite()) {
            throw new IllegalArgumentException("The given path [" + path + "] is not writable.");
        }
        return path;
    }

    /**
     * This method retrieves a DocumentIndex with the specified documentId (calculated from the wordSourceId) from the
     * persistence. It does return <tt>null</tt>, if DocumentIndex with the given WordSourceId is not found in the
     * persistence.
     * <p/>
     * CS section 1.4.4.1 describes the algorithm of this method.
     *
     * @param documentId id of document to retrieve
     *
     * @return the retrieved DocumentIndex, or <tt>null</tt> if document index with given WordSourceId is not found in
     *         the persistence
     *
     * @throws IllegalArgumentException  when wordSourceId is <tt>null</tt>
     * @throws IndexPersistenceException when the implementation fails to retrieve document index with the specified
     *                                   WordSourceId
     */
    private DocumentIndex getDocumentIndex(final String documentId) throws IndexPersistenceException {
        final File[] indexFiles = new File(persistenceDirPath).listFiles(
            new PrefixSuffixFilter(DOC_INDEX_FILE_NAME_PREFIX + documentId, ".xml"));

        if (indexFiles == null) {
            throw new IndexPersistenceException("An error has occurred while scanning the persistence directory ["
                + persistenceDirPath + "] for matching persistence files.");
        }
        if (indexFiles.length == 0) {
            return null;
        }

        final DocumentIndexHandler dh = new DocumentIndexHandler();
        for (int i = 0; i < indexFiles.length; i++) {
            final String expectedFileName =
                DOC_INDEX_FILE_NAME_PREFIX + documentId + "_" + (i + 1) + ".xml";
            final File currentFile = new File(persistenceDirPath, expectedFileName);
            if (!currentFile.exists()) {
                throw new IndexPersistenceException(
                    "Corrupt Persistence detected: Expected to encounter file with name [" + expectedFileName
                        + "], but next file found has name [" + currentFile.getName() + "].");

            }
            saxParse(currentFile, dh);
        }

        return dh.createDocumentIndex();
    }

    /**
     * This method writes the actual document index data files.
     *
     * @param header        the header XML-fragment string to be used when starting a new file
     * @param footer        the footer XML-fragment string to be used when ending a file
     * @param documentID    the document id calculated from the document index
     * @param documentIndex the index to be written to the files
     *
     * @throws IndexPersistenceException in case the operation fails
     */
    private void writeDocumentDataFiles(final String header, final String footer, final String documentID,
                                        final DocumentIndex documentIndex)
        throws
        IndexPersistenceException {
        OutputStream outputStream = null;
        try {
            final byte[] headerData = header.getBytes(UTF_8_CHARSET);
            final byte[] footerData = footer.getBytes(UTF_8_CHARSET);

            //start the initial file here
            int fileSequence = 1;
            outputStream = new BufferedOutputStream(
                new FileOutputStream(new File(persistenceDirPath, DOC_INDEX_FILE_NAME_PREFIX + documentID + "_"
                    + (fileSequence++) + ".xml")));
            outputStream.write(headerData);
            int bytesWritten = headerData.length;

            // iterate the words and write them out
            final Set set = documentIndex.getWords().entrySet();
            for (Iterator iterator = set.iterator(); iterator.hasNext();) {
                final Map.Entry entry = (Map.Entry) iterator.next();
                final List indices = (List) entry.getValue();
                final StringBuffer wordData = new StringBuffer();
                wordData.append("<word-index><word>");
                wordData.append(escapeForXML(((CollationKey) entry.getKey()).getSourceString()));
                wordData.append("</word>");
                for (Iterator iterator1 = indices.iterator(); iterator1.hasNext();) {
                    wordData.append("<pos>");
                    wordData.append((Integer) iterator1.next());
                    wordData.append("</pos>");
                }
                wordData.append("</word-index>");
                final byte[] wordEntry = wordData.toString().getBytes(UTF_8_CHARSET);

                // we have to start a new file in case
                // the data to be written would exceed the file size threshold
                if (bytesWritten + wordEntry.length + footerData.length >= fileSizeLimit - fileSizeThreshold) {
                    //close previous file
                    outputStream.write(footerData);
                    outputStream.close();

                    //create new file
                    outputStream = new BufferedOutputStream(new FileOutputStream(new File(persistenceDirPath,
                        DOC_INDEX_FILE_NAME_PREFIX + documentID + "_"
                            + (fileSequence++) + ".xml")));

                    //write header
                    outputStream.write(headerData);
                    bytesWritten = headerData.length;
                }

                //now write the word entry data to current file
                outputStream.write(wordEntry);
                bytesWritten += wordEntry.length;
            }
            outputStream.write(footerData);
            outputStream.flush();
        } catch (IOException e) {
            throw new IndexPersistenceException("Error while persisting index [" + documentIndex + "] to file.", e);
        } finally {
            safeClose(outputStream);
        }
    }

    /**
     * This method closes the given output stream, ignoring all exceptions that may occur during this operation .
     *
     * @param outputStream the stream to be closed, may be <tt>null</tt>
     */
    private static void safeClose(final OutputStream outputStream) {
        if (outputStream != null) {
            try {
                outputStream.close();
            } catch (IOException e) {
                // There is nothing we can do here, as we may have come here due to an exception, we
                // cannot re-throw this one as it would mask the original exception.
            }
        }
    }

    /**
     * This method creates a new SAXParser and lets it parse the given file using the given DefaultHandler.
     *
     * @param file           the file to be parsed
     * @param defaultHandler the handler to be used when parsing
     *
     * @throws IndexPersistenceException in case the parse operation fails
     * @throws IllegalArgumentException  incase any arg is <tt>null</tt>
     */
    private static void saxParse(final File file, final DefaultHandler defaultHandler)
        throws IndexPersistenceException {
        if (file == null) {
            throw new IllegalArgumentException("The parameter named [file] was null.");
        }
        if (defaultHandler == null) {
            throw new IllegalArgumentException("The parameter named [defaultHandler] was null.");
        }

        final SAXParser saxParser;
        try {
            saxParser = SAXParserFactory.newInstance().newSAXParser();
        } catch (ParserConfigurationException e) {
            throw new IndexPersistenceException("Error while creating SAX parser.", e);
        } catch (SAXException e) {
            throw new IndexPersistenceException("Error while creating SAX parser.", e);
        }
        try {
            saxParser.parse(file, defaultHandler);
        } catch (SAXException e) {
            throw new IndexPersistenceException("Error while parsing input file [" + file.getName() + "] .",
                e);

        } catch (IOException e) {
            throw new IndexPersistenceException("Error while parsing input file [" + file.getName() + "] .",
                e);
        }
    }

    /**
     * This utility method retrieves an attribute value from the given attributes list, throwing a SAXException in case
     * the attribute does not exist.
     *
     * @param attributes the attributes list to retrieve the value from
     * @param attrName   the attribute name
     * @param tagName    the current tag name, used in error messages
     *
     * @return the attribute value
     *
     * @throws SAXException in case the attribute does not exist
     */
    private static String getAttribute(final Attributes attributes, final String attrName, final String tagName)
        throws SAXException {
        final String attributeValue = attributes.getValue(attrName);
        if (attributeValue == null) {
            throw new SAXException(
                "The required attribute [" + attrName + "] of tag [" + tagName + "] was missing.");
        }
        return attributeValue;
    }

    /**
     * This method writes the document_usage.xml from the given map, which is expected to contain values of type {@link
     * DocumentUsage}. All these values are written to the file.
     *
     * @param documentUsages the document usages map to be written
     *
     * @throws IndexPersistenceException in case the operation fails
     */
    private void writeDocumentUsages(final Map documentUsages) throws IndexPersistenceException {
        final StringBuffer fileContent = new StringBuffer();
        final Collection usages = documentUsages.values();
        fileContent.append("<?xml version=\"1.0\" encoding=\"" + UTF_8_CHARSET + "\" standalone=\"yes\"?><documents>");
        for (Iterator iterator = usages.iterator(); iterator.hasNext();) {
            final DocumentUsage documentUsage = (DocumentUsage) iterator.next();
            fileContent.append("<document-usage document-id=\"");
            fileContent.append(escapeForXML(documentUsage.getDocumentId()));
            fileContent.append("\" usage-count=\"");
            fileContent.append(documentUsage.getUsageCount());
            fileContent.append("\"><collection-inclusion>");
            final String[] collectionIds = documentUsage.getCollectionIds();
            for (int i = 0; i < collectionIds.length; i++) {
                final String id = collectionIds[i];
                fileContent.append("<collection collection-id=\"");
                fileContent.append(escapeForXML(id));
                fileContent.append("\"/>");
            }
            fileContent.append("</collection-inclusion>");
            fileContent.append("</document-usage>");
        }
        fileContent.append("</documents>");
        writeUTF8File(fileContent.toString(), new File(persistenceDirPath, DOC_USAGE_FILE_NAME));
    }

    /**
     * This method writes the given String to the given file, encoding the content as UTF-8.
     *
     * @param fileContent     the content to be written
     * @param persistenceFile the file to write to
     *
     * @throws IndexPersistenceException in case the operation fails
     */
    private static void writeUTF8File(final String fileContent, final File persistenceFile)
        throws IndexPersistenceException {
        OutputStream outputStream = null;
        try {
            final byte[] fileData = fileContent.getBytes(UTF_8_CHARSET);
            outputStream = new FileOutputStream(persistenceFile);
            outputStream.write(fileData);
        } catch (IOException e) {
            throw new IndexPersistenceException(
                "Error while writing persistence file [" + persistenceFile.getName() + "].", e);
        } finally {
            safeClose(outputStream);
        }
    }

    /**
     * This method does parse the document_usage.xml file. The content is returned as a map containing String keys being
     * the documentIds and values of type {@link DocumentUsage}.
     *
     * @return the map representing the content of the document_usage.xml file
     *
     * @throws IndexPersistenceException in case the operation fails
     */
    private Map readDocumentUsages() throws IndexPersistenceException {
        final File docUsageFile = new File(persistenceDirPath, DOC_USAGE_FILE_NAME);
        if (docUsageFile.exists()) {
            final DocumentUsageHandler dh = new DocumentUsageHandler();
            saxParse(docUsageFile, dh);
            return dh.getDocumentUsages();
        } else {
            throw new IndexPersistenceException("The document usage file,which was expected to exist at ["
                + docUsageFile.getAbsolutePath() + "] did not exist.");
        }
    }

    /**
     * Replace characters having special meaning in XML with their escaped equivalents, using character entities such as
     * <tt>'&amp;amp;'</tt> or numeric entities such as<tt>'&amp;#12;'</tt>.
     * <p/>
     * The filtering whether a character needs to be escaped is done with respect to the fact that the output is written
     * as UTF-8 encoding, i.e. two-byte characters do not need to be escaped.
     * <p/>
     * See: <ul><li>http://www.w3.org/TR/2000/REC-xml-20001006#syntax</li>
     * <p/>
     * <li>http://www.w3.org/TR/2000/REC-xml-20001006#sec-predefined-ent</li>
     * <p/>
     * <li>http://www.w3.org/TR/2000/REC-xml-20001006#sec-entexpand</li></ul>
     *
     * @param string the String to be escaped
     *
     * @return the escaped version of the given String
     */
    private static String escapeForXML(final String string) {
        final StringBuffer result = new StringBuffer();

        final StringCharacterIterator iterator = new StringCharacterIterator(string);
        char ch = iterator.current();
        while (ch != StringCharacterIterator.DONE) {
            switch (ch) {
            case '<':
                result.append("&lt;");
                break;
            case '>':
                result.append("&gt;");
                break;
            case '"':
                result.append("&quot;");
                break;
            case '\'':
                result.append("&apos;");
                break;
            case '&':
                result.append("&amp;");
                break;
            default:
                // If the character is not printable, print as character reference.
                // Non printables are below ASCII space
                if (ch < ' ' || ch == '\u00f7') {
                    result.append("&#");
                    result.append(Integer.toString(ch));
                    result.append(';');
                } else {
                    result.append(ch);
                }
            }

            ch = iterator.next();
        }
        return result.toString();
    }

    /**
     * This is a simple file filter that matches all files with the given prefix and suffix.
     */
    private static class PrefixSuffixFilter implements FileFilter {
        /**
         * This is the prefix to be matched by the filter.
         */
        private final String prefix;
        /**
         * This is the suffix to be matched by the filter.
         */
        private final String suffix;

        /**
         * Creates new PrefixSuffixFilter that matches the given prefix and suffix.
         *
         * @param prefix the prefix to be matched
         * @param suffix the suffix to be matched
         *
         * @throws IllegalArgumentException if any arg is <tt>null</tt>
         */
        private PrefixSuffixFilter(final String prefix, final String suffix) {
            if (prefix == null) {
                throw new IllegalArgumentException("The parameter named [prefix] was null.");
            }
            if (suffix == null) {
                throw new IllegalArgumentException("The parameter named [suffix] was null.");
            }

            this.prefix = prefix;
            this.suffix = suffix;
        }

        /**
         * Tests whether or not the specified abstract path name should be included in a path name list.
         *
         * @param pathname The abstract path name to be tested
         *
         * @return <code>true</code> if and only if <code>pathname</code> should be included
         */
        public boolean accept(final File pathname) {
            final String name = pathname.getName();
            return name.startsWith(prefix) && name.endsWith(suffix);
        }
    }

    /**
     * This DefaultHandler instance is used to parse document index XML files.
     */
    private static class DocumentIndexHandler extends WordSourceIdHandler {
        /**
         * This is a state flag which signals whether we are between the start and the end of an <em>word</em>. This
         * information is used in {@link #characters(char[], int, int)} to determine whether the characters are child
         * element of that element or not.
         */
        private boolean inWord = false;
        /**
         * This is a state flag which signals whether we are between the start and the end of an <em>pos</em>. This
         * information is used in {@link #characters(char[], int, int)} to determine whether the characters are child
         * element of that element or not.
         */
        private boolean inPos = false;

        /**
         * This is the collator instance matching the document index's locale. It is used for converting words to {@link
         * CollationKey}s. as this instance is constructed after the <em>word-source-data</em> element has ended, the
         * case that the field is not <tt>null</tt> any more does furthermore signal that a <em>word-source-data</em>
         * element has been parsed fully by the super class and thus no more event must be delegated to the super
         * class.
         */
        private Collator collator = null;

        /**
         * This field contains the parsed words of the index as a map from {@link java.text.CollationKey} to a {@link
         * java.util.List} of {@link Integer}s being the indices.
         */

        private Map words = new HashMap();

        /**
         * This field contains the currently parsed word, if any.
         */
        private String currentWord = null;
        /**
         * This field contains the currently parsed indices, if any.
         */
        private List currentIndices = null;
        /**
         * This field contains the currently parsed pos value.
         */
        private String currentPos = null;

        /**
         * This method is called by the parser when a document start element is encountered.
         *
         * @param uri        the uri of the element encountered
         * @param localName  the local name of the element encountered
         * @param qName      the qualified name of the element encountered
         * @param attributes the attributes of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void startElement(final String uri, final String localName, final String qName,
                                 final Attributes attributes)
            throws SAXException {
            //delegate to super class as long as no full word-source-data element has been parsed
            if (collator == null) {
                super.startElement(uri, localName, qName, attributes);
            }

            if ("pos".equals(qName)) {
                inPos = true;
            } else if ("word".equals(qName)) {
                inWord = true;
            } else if ("word-index".equals(qName)) {
                if (collator == null) {
                    throw new SAXException(
                        "There was no [word-source-data] specified prior to the [word-index] element.");
                }
                currentIndices = new ArrayList();
            }
        }

        /**
         * This method is called by the parser when a TEXT element is encountered.
         *
         * @param ch     the characters encountered
         * @param start  the start index in ch
         * @param length the length in ch
         *
         * @throws SAXException in case some error occurs
         */
        public void characters(final char[] ch, final int start, final int length) throws SAXException {
            //delegate to super class as long as no full word-source-data element has been parsed
            if (collator == null) {
                super.characters(ch, start, length);
            }

            if (inPos) {
                final String s = new String(ch, start, length);
                currentPos = currentPos == null ? s : currentPos + s;
            } else if (inWord) {
                final String s = new String(ch, start, length);
                currentWord = currentWord == null ? s : currentWord + s;
            }
        }

        /**
         * This method is called by the parser when a document end element is encountered.
         *
         * @param uri       the uri of the element encountered
         * @param localName the local name of the element encountered
         * @param qName     the qualified name of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void endElement(final String uri, final String localName, final String qName) throws SAXException {
            // delegate to super class as long as no full word-source-data element has been parsed
            if (collator == null) {
                super.endElement(uri, localName, qName);
            }

            if ("pos".equals(qName)) {
                if (currentPos == null) {
                    throw new SAXException(
                        "The child text element of tag [pos] was expected to contain non-whitespace data.");
                }
                try {
                    currentIndices.add(Integer.valueOf(currentPos.trim()));
                } catch (NumberFormatException e) {
                    throw new SAXException(
                        "The found [pos] tag value [" + currentPos + "] was not parseable into a valid int.", e);
                }
                inPos = false;
                currentPos = null;
            } else if ("word".equals(qName)) {
                if (currentWord == null) {
                    throw new SAXException(
                        "The child text element of tag [word] was expected to contain non-whitespace data.");
                }
                inWord = false;
            } else if ("word-index".equals(qName)) {
                // full entry ended, put it to map.
                words.put(collator.getCollationKey(currentWord), currentIndices);
                currentWord = null;
            } else if ("word-source-data".equals(qName)) {
                //super class has done its job, now we can create our collator.
                try {
                    collator = Collator.getInstance(createWordSourceId().getSourceLocale());
                } catch (IndexPersistenceException e) {
                    throw new SAXException("document index header contained unexpected data.", e);
                }
            }
        }

        /**
         * This method creates a DocumentIndex instance from the data encountered during parsing.
         *
         * @return the instance created
         *
         * @throws IndexPersistenceException in case not enough data for creating the instance has been encountered
         *                                   during parsing or the creation of the instance fails
         */
        private DocumentIndex createDocumentIndex() throws IndexPersistenceException {
            final WordSourceId sourceId = createWordSourceId();
            return new DocumentIndex(sourceId, words);
        }
    }

    /**
     * This is a default handler for parsing WordSourceIds.
     */
    private static class WordSourceIdHandler extends DefaultHandler {
        /**
         * This is a state flag which signals whether we are between the start and the end of an
         * <em>source-identity</em>. This information is used in {@link #characters(char[], int, int)} to determine
         * whether the characters are child element of that element or not.
         */
        private boolean inSourceIdentity = false;
        /**
         * This is a state flag which signals whether we are between the start and the end of an <em>class-name</em>.
         * This information is used in {@link #characters(char[], int, int)} to determine whether the characters are
         * child element of that element or not.
         */
        private boolean inClassName = false;

        /**
         * This is a state flag which signals whether we are between the start and the end of an
         * <em>word-source-data</em>. This information is used as an optimization that will disable a lot of unnecessary
         * comparison in case the parser is outside the parent element this instance is responsible for.
         */
        private boolean inScope = false;

        /**
         * This field contains the parsed value of the document index source identity.
         */
        private String sourceIdentityValue = null;
        /**
         * This field contains the parsed value of the document index identity class name.
         */
        private String classNameValue = null;

        /**
         * This field contains the parsed value of the document index identity locale.
         */
        private Locale locale = null;

        /**
         * This field contains the parsed value of the document index identity delimiters.
         */
        private List delimiters = new ArrayList();

        /**
         * This method is called by the parser when a document start element is encountered.
         *
         * @param uri        the uri of the element encountered
         * @param localName  the local name of the element encountered
         * @param qName      the qualified name of the element encountered
         * @param attributes the attributes of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void startElement(final String uri, final String localName, final String qName,
                                 final Attributes attributes)
            throws SAXException {
            if (inScope) {
                if ("source-identity".equals(qName)) {
                    inSourceIdentity = true;
                } else if ("class-name".equals(qName)) {
                    inClassName = true;
                } else if ("locale".equals(qName) && locale == null) {
                    final String language = getAttribute(attributes, "language", "locale");
                    final String country = getAttribute(attributes, "country", "locale");
                    final String variant = getAttribute(attributes, "variant", "locale");
                    locale = new Locale(language, country, variant);
                } else if ("entry".equals(qName)) {
                    delimiters.add(getAttribute(attributes, "delimiter", "entry"));
                }
            } else if ("word-source-data".equals(qName)) {
                inScope = true;
            }
        }

        /**
         * This method is called by the parser when a TEXT element is encountered.
         *
         * @param ch     the characters encountered
         * @param start  the start index in ch
         * @param length the length in ch
         *
         * @throws SAXException in case some error occurs
         */
        public void characters(final char[] ch, final int start, final int length) throws SAXException {
            if (inScope) {
                final String s = new String(ch, start, length);
                if (inSourceIdentity) {
                    sourceIdentityValue = sourceIdentityValue == null ? s : sourceIdentityValue + s;
                } else if (inClassName) {
                    classNameValue = classNameValue == null ? s : classNameValue + s;
                }
            }
        }

        /**
         * This method is called by the parser when a document end element is encountered.
         *
         * @param uri       the uri of the element encountered
         * @param localName the local name of the element encountered
         * @param qName     the qualified name of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void endElement(final String uri, final String localName, final String qName) throws SAXException {
            if (inScope) {
                if ("source-identity".equals(qName)) {
                    if (sourceIdentityValue == null) {
                        throw new SAXException(
                            "The child text element of tag [source-identity] was expected "
                                + "to contain non-whitespace data.");
                    }
                    inSourceIdentity = false;
                } else if ("class-name".equals(qName)) {
                    if (classNameValue == null) {
                        throw new SAXException(
                            "The child text element of tag [class-name] was expected to contain non-whitespace data.");
                    }
                    inClassName = false;
                } else if ("delimiters".equals(qName)) {
                    if (delimiters.isEmpty()) {
                        throw new SAXException("At least one [entry] element is expected in the [delimiters] element.");
                    }
                } else if ("word-source-data".equals(qName)) {
                    if (sourceIdentityValue == null) {
                        throw new SAXException(
                            "There was no [source-identity] element in the [word-source-data] element.");
                    }
                    if (classNameValue == null) {
                        throw new SAXException("There was no [class-name] element in the [word-source-data] element.");
                    }
                    if (locale == null) {
                        throw new SAXException("There was no [locale] element in the [word-source-data] element.");
                    }
                    inScope = false;
                }

            }
        }

        /**
         * This method creates a WordSourceId from the data encountered during the parse process.
         *
         * @return the WordSourceId parsed
         *
         * @throws IndexPersistenceException in case not enough data for creating the instance has been encountered
         *                                   during parsing or the creation of the instance fails
         */
        public WordSourceId createWordSourceId() throws IndexPersistenceException {
            if (sourceIdentityValue == null) {
                throw new IndexPersistenceException("No [source-identity] value encountered during parsing.");
            }
            if (classNameValue == null) {
                throw new IndexPersistenceException("No [class-name] value encountered during parsing.");
            }
            if (classNameValue.trim().length() == 0) {
                throw new IndexPersistenceException("Empty [class-name] value encountered during parsing.");
            }
            if (locale == null) {
                throw new IndexPersistenceException("No [locale] value encountered during parsing.");
            }
            if (delimiters.isEmpty()) {
                throw new IndexPersistenceException("No [delimiters] value encountered during parsing.");
            }

            //decode the sourceIdentity object from the Base64 String
            final Base64Decoder base64Decoder = new Base64Decoder(Base64Codec.IGNORE_ALL, null,
                Base64Codec.STANDARD_ALPHABET);
            final byte[] bytes = sourceIdentityValue.getBytes();
            final byte[] out = new byte[bytes.length];
            base64Decoder.setInput(bytes);
            int byteCount = 0;
            try {
                byteCount = base64Decoder.inflate(out);
            } catch (DataFormatException e) {
                throw new IndexPersistenceException(
                    "Error while decoding the Base64 data found in the [source-identity] tag.", e);
            }
            final byte[] serializedData = new byte[byteCount];
            System.arraycopy(out, 0, serializedData, 0, byteCount);
            final Object sourceIdentity = Utils.deSerializeObject(serializedData);

            // list to array
            final String[] delims = (String[]) delimiters.toArray(new String[delimiters.size()]);

            // build the actual WordSourceId
            return new WordSourceId(sourceIdentity, classNameValue, delims, locale);
        }
    }

    /**
     * This is a DefaultHandler for parsing CollectionIndex instances.
     */
    private static class DocumentCollectionHandler extends DefaultHandler {
        /**
         * This is a state flag which signals whether we are between the start and the end of an <em>document-id</em>.
         * This information is used in {@link #characters(char[], int, int)} to determine whether the characters are
         * child element of that element or not.
         */
        private boolean inDocumentId = false;
        /**
         * This field contains the parsed document ids.
         */

        private List documentIds = new ArrayList();

        /**
         * This field contains the currently parsed document, if any.
         */
        private String currentDocumentId = null;

        /**
         * This method is called by the parser when a document start element is encountered.
         *
         * @param uri        the uri of the element encountered
         * @param localName  the local name of the element encountered
         * @param qName      the qualified name of the element encountered
         * @param attributes the attributes of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void startElement(final String uri, final String localName, final String qName,
                                 final Attributes attributes)
            throws SAXException {
            if ("document-id".equals(qName)) {
                inDocumentId = true;
            }
        }

        /**
         * This method is called by the parser when a TEXT element is encountered.
         *
         * @param ch     the characters encountered
         * @param start  the start index in ch
         * @param length the length in ch
         *
         * @throws SAXException in case some error occurs
         */
        public void characters(final char[] ch, final int start, final int length) throws SAXException {
            if (inDocumentId) {
                final String s = new String(ch, start, length);
                currentDocumentId = currentDocumentId == null ? s : currentDocumentId + s;
            }
        }

        /**
         * This method is called by the parser when a document end element is encountered.
         *
         * @param uri       the uri of the element encountered
         * @param localName the local name of the element encountered
         * @param qName     the qualified name of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void endElement(final String uri, final String localName, final String qName) throws SAXException {
            if ("document-id".equals(qName)) {
                if (currentDocumentId == null) {
                    throw new SAXException(
                        "The child text element of tag [document-id] was expected to contain non-whitespace data.");
                }
                documentIds.add(currentDocumentId.trim());
                inDocumentId = false;
                currentDocumentId = null;
            }
        }

        /**
         * This method returns all document ids found during parsing the document collection.
         *
         * @return the document ids found during parsing
         */
        private String[] getDocumentIds() {
            return (String[]) documentIds.toArray(new String[documentIds.size()]);
        }
    }

    /**
     * This is the DefaultHandler used for parsing the document_usage.xml.
     */
    private static class DocumentUsageHandler extends DefaultHandler {
        /**
         * This field contains the parsed usages for documents.
         */

        private Map usages = new HashMap();

        /**
         * This field contains the currently parsed document, if any.
         */
        private DocumentUsage currentDocumentUsage = null;

        /**
         * This method is called by the parser when a document start element is encountered.
         *
         * @param uri        the uri of the element encountered
         * @param localName  the local name of the element encountered
         * @param qName      the qualified name of the element encountered
         * @param attributes the attributes of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void startElement(final String uri, final String localName, final String qName,
                                 final Attributes attributes)
            throws SAXException {
            if ("document-usage".equals(qName)) {
                final String documentId = getAttribute(attributes, "document-id", "document-usage");
                final String countStr = getAttribute(attributes, "usage-count", "document-usage");
                final int count;
                try {
                    count = Integer.parseInt(countStr.trim());
                } catch (NumberFormatException e) {
                    throw new SAXException("The found [count] tag attribute value ["
                        + countStr + "] was not parseable into a valid int.", e);
                }
                currentDocumentUsage = new DocumentUsage(documentId, count);
            } else if ("collection".equals(qName)) {
                if (currentDocumentUsage == null) {
                    throw new SAXException("Illegal structure encountered, expected a [document-usage] "
                        + "tag to exist as parent of a [collection] tag.");
                }
                currentDocumentUsage.add(getAttribute(attributes, "collection-id", "collection"));
            }
        }

        /**
         * This method is called by the parser when a document end element is encountered.
         *
         * @param uri       the uri of the element encountered
         * @param localName the local name of the element encountered
         * @param qName     the qualified name of the element encountered
         *
         * @throws SAXException in case some error occurs
         */
        public void endElement(final String uri, final String localName, final String qName) throws SAXException {
            if ("document-usage".equals(qName)) {
                usages.put(currentDocumentUsage.getDocumentId(), currentDocumentUsage);
                currentDocumentUsage = null;
            }
        }

        /**
         * This method does return the document usages parsed from the file.
         *
         * @return the document usages parsed from the file as map from String to {@link DocumentUsage}
         */
        private Map getDocumentUsages() {
            return usages;
        }
    }

    /**
     * This class represents the document usage elements in  document_usage.xml.
     */
    private static final class DocumentUsage {
        /**
         * This is the document id of the usage information.
         */
        private final String documentId;
        /**
         * This is the use count of the usage information.
         */
        private int usageCount;
        /**
         * This is the collection of document index collections the document is contained in.
         */
        private Set collectionIds = new HashSet();

        /**
         * Creates a new DocumentUsage from the given arguments.
         *
         * @param documentId the document id for the instance
         * @param usageCount the usage count for this instance
         *
         * @throws IllegalArgumentException in case the document id is <tt>null</tt> or empty String
         */
        private DocumentUsage(final String documentId, final int usageCount) {
            if (documentId == null) {
                throw new IllegalArgumentException("The parameter named [documentId] was null.");
            }
            if (documentId.trim().length() == 0) {
                throw new IllegalArgumentException("The parameter named [documentId] was an empty String.");
            }

            this.documentId = documentId;
            this.usageCount = usageCount;
        }

        /**
         * Creates a new DocumentUsage from the given argument. The use count is initialized to 0.
         *
         * @param documentId the document id for the instance
         *
         * @throws IllegalArgumentException in case the document id is <tt>null</tt> or empty String
         */
        private DocumentUsage(final String documentId) {
            this.documentId = documentId;
            usageCount = 0;
        }

        /**
         * This method adds the given collection id to the document usage.
         *
         * @param collectionId the collection id to be removed
         */
        private void add(final String collectionId) {
            collectionIds.add(collectionId);
        }

        /**
         * This method removes the given collection id from the document usage.
         *
         * @param collectionId the collection id to be removed
         */
        private void remove(final String collectionId) {
            collectionIds.remove(collectionId);
        }

        /**
         * This method returns the collection ids contained in this document usage.
         *
         * @return the collection ids contained in this document usage
         */
        private String[] getCollectionIds() {
            return (String[]) collectionIds.toArray(new String[collectionIds.size()]);
        }

        /**
         * This method returns the document id of this document usage.
         *
         * @return the document id of this document usage
         */
        private String getDocumentId() {
            return documentId;
        }

        /**
         * This method returns the use count of this document usage.
         *
         * @return the use count of this document usage
         */
        private int getUsageCount() {
            return usageCount;
        }

        /**
         * This method increments the usage count of this document usage.
         */
        private void incUsageCount() {
            usageCount++;
        }

        /**
         * This method decrements the usage count of this document usage.
         */
        private void decUsageCount() {
            usageCount--;
        }
    }
}
