/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.document.index.CollectionIndex;
import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.impl.Utils;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.UnknownNamespaceException;
import junit.framework.TestCase;
import org.custommonkey.xmlunit.Diff;
import org.custommonkey.xmlunit.XMLUnit;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.Connection;
import java.text.CollationKey;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;


/**
 * This class is the base class for all unit test classes for the Document Indexer Persistence component, that depend on
 * properly configured environment. It performs the setup necessary for the Document Indexer Persistence component to
 * work properly.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class BaseDocumentIndexerPersistenceTest extends TestCase {
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} namespace of the simple DB Document Indexer
     * Persistence configuration.
     */
    protected static final String SIMPLE_DB_CONFIGURATION_NAMESPACE = "com.topcoder.document.indexer.db.simple";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} namespace of the fast DB Document Indexer Persistence
     * configuration.
     */
    protected static final String FAST_DB_CONFIGURATION_NAMESPACE = "com.topcoder.document.indexer.db.fast";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} namespace of the XML Document Indexer Persistence
     * configuration.
     */
    protected static final String XML_CONFIGURATION_NAMESPACE = "com.topcoder.document.indexer.xml";
    /**
     * This is the default {@link com.topcoder.util.config.ConfigManager} namespace of the DBConnection factory.
     */
    protected static final String DEFAULT_DB_CONNECTION_FACTORY_NAMESPACE
        = "com.topcoder.db.connectionfactory.DBConnectionFactoryImpl";

    /**
     * This is the name of the default Document Indexer Persistence configuration file.
     */
    protected static final String DEFAULT_PERSISTENCE_CONFIGURATION = "defaultConfig.xml";
    /**
     * This is the default persistence directory.
     */
    protected static final String DEFAULT_PERSISTENCE_DIR_PATH = "test_files/xmlPersistence";

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        loadFullConfiguration(BaseDocumentIndexerPersistenceTest.DEFAULT_PERSISTENCE_CONFIGURATION);
    }

    /**
     * This method does the cleanup after test run.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        clearConfig();
    }

    /**
     * This method removes all Document Indexer Persistence -specific configuration namespaces plus the namespaces of
     * dependant components from the ConfigManager singleton of this class' classloader and then loads the configuration
     * with the given filename/classloader path into the ConfigManager instance.
     *
     * @param filename the config file to be loaded as new Document Indexer Persistence configuration
     *
     * @throws Exception                in case some unexpected Exception occurs
     * @throws IllegalArgumentException in case the given filename is <tt>null</tt>
     */
    protected void loadFullConfiguration(final String filename) throws Exception {
        if (filename == null) {
            throw new IllegalArgumentException("The parameter named [filename] was null");
        }

        // clear config manager data
        clearConfig();

        // load the configuration for component,
        ConfigManager.getInstance().add("configurations/" + filename);
    }

    /**
     * This method removes the Document Indexer Simple Persistence configuration namespace from the ConfigManager
     * singleton of this class' classloader and then loads the configuration with the given filename/classloader path
     * into the ConfigManager instance.
     *
     * @param filename the config file to be loaded as new Document Indexer Simple Persistence configuration
     *
     * @throws Exception                in case some unexpected Exception occurs
     * @throws IllegalArgumentException in case the given filename is <tt>null</tt>
     */
    protected void loadSimpleConfiguration(final String filename) throws Exception {
        if (filename == null) {
            throw new IllegalArgumentException("The parameter named [filename] was null");
        }

        // clear config manager data
        removeNamespace(SIMPLE_DB_CONFIGURATION_NAMESPACE);

        // load the configuration for component,
        ConfigManager.getInstance().add("configurations/" + filename);
    }

    /**
     * This method removes the Document Indexer Fast Persistence configuration namespace from the ConfigManager
     * singleton of this class' classloader and then loads the configuration with the given filename/classloader path
     * into the ConfigManager instance.
     *
     * @param filename the config file to be loaded as new Document Indexer Fast Persistence configuration
     *
     * @throws Exception                in case some unexpected Exception occurs
     * @throws IllegalArgumentException in case the given filename is <tt>null</tt>
     */
    protected void loadFastConfiguration(final String filename) throws Exception {
        if (filename == null) {
            throw new IllegalArgumentException("The parameter named [filename] was null");
        }

        // clear config manager data
        removeNamespace(FAST_DB_CONFIGURATION_NAMESPACE);

        // load the configuration for component,
        ConfigManager.getInstance().add("configurations/" + filename);
    }

    /**
     * This method removes the Document Indexer XML Persistence configuration namespace from the ConfigManager singleton
     * of this class' classloader and then loads the configuration with the given filename/classloader path into the
     * ConfigManager instance.
     *
     * @param filename the config file to be loaded as new Document Indexer XML Persistence configuration
     *
     * @throws Exception                in case some unexpected Exception occurs
     * @throws IllegalArgumentException in case the given filename is <tt>null</tt>
     */
    protected void loadXmlConfiguration(final String filename) throws Exception {
        if (filename == null) {
            throw new IllegalArgumentException("The parameter named [filename] was null");
        }

        // clear config manager data
        removeNamespace(XML_CONFIGURATION_NAMESPACE);

        // load the configuration for component,
        ConfigManager.getInstance().add("configurations/" + filename);
    }

    /**
     * This method removes all namespaces that are used by the Document Indexer Persistence component in testcases from
     * the ConfigManager singleton instance of this class' classloader.
     */
    protected void clearConfig() {
        removeNamespace(BaseDocumentIndexerPersistenceTest.SIMPLE_DB_CONFIGURATION_NAMESPACE);
        removeNamespace(BaseDocumentIndexerPersistenceTest.FAST_DB_CONFIGURATION_NAMESPACE);
        removeNamespace(BaseDocumentIndexerPersistenceTest.XML_CONFIGURATION_NAMESPACE);
        removeNamespace(BaseDocumentIndexerPersistenceTest.DEFAULT_DB_CONNECTION_FACTORY_NAMESPACE);
    }

    /**
     * This method removes the given namespace from the ConfigManager singleton instance of this class' classloader.
     *
     * @param namespace the namespace to be removed from the ConfigManager, must not be <tt>null</tt>
     *
     * @throws IllegalArgumentException in case the given namespace was <tt>null</tt>
     */
    private void removeNamespace(final String namespace) {
        if (namespace == null) {
            throw new IllegalArgumentException("The parameter named [namespace] was null");
        }

        final ConfigManager cm = ConfigManager.getInstance();
        if (cm.existsNamespace(namespace)) {
            try {
                cm.removeNamespace(namespace);
            } catch (UnknownNamespaceException ignored) {
                // This may happen if the config namespace to be removed did not exist
                // (which only can occur because of threading issues, as we check for
                // namespace existence before removing), but as the result is the
                // desired ConfigManager state (the namespace not existing any more
                // in CM), we can ignore this
            }
        }
    }

    /**
     * This method clears the simple database used in test cases.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void clearSimpleDatabase() throws Exception {
        loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        final AbstractDBIndexPersistence dbIndexPersistence = new SimpleAccessDBIndexPersistence(
            SIMPLE_DB_CONFIGURATION_NAMESPACE);
        final Connection connection = dbIndexPersistence.getConnection();
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM WORD_POSITION", new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM WORD", new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DELIMITER", new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DOCUMENT_COLLECTION_DOCUMENT_XREF",
            new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DOCUMENT_COLLECTION", new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DOCUMENT", new Object[0]);
        dbIndexPersistence.commitTransaction(connection);
    }

    /**
     * This method clears the fast database used in test cases.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void clearFastDatabase() throws Exception {
        loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        final AbstractDBIndexPersistence dbIndexPersistence = new FastAccessDBIndexPersistence(
            FAST_DB_CONFIGURATION_NAMESPACE);
        final Connection connection = dbIndexPersistence.getConnection();
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM WORD_DATA", new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DELIMITER", new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DOCUMENT_COLLECTION_DOCUMENT_XREF",
            new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DOCUMENT_COLLECTION", new Object[0]);
        DatabaseUtils.doDMLQuery(connection, dbIndexPersistence, "DELETE FROM DOCUMENT", new Object[0]);
        dbIndexPersistence.commitTransaction(connection);
    }

    /**
     * This method clears the XML persistence dir used in test cases.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void clearXMLPersistence() throws Exception {
        loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        final String path = Utils.lookupValidStringFromConfigManager(XML_CONFIGURATION_NAMESPACE, "XmlPersistencePath");
        final File databaseDir = new File(path);
        final File[] files = databaseDir.listFiles();
        for (int i = 0; i < files.length; i++) {
            files[i].delete();
        }

        final File sourcedir = new File("test_files/emptyXMLDatabase");
        final File[] sourceFiles = sourcedir.listFiles();
        for (int i1 = 0; i1 < sourceFiles.length; i1++) {
            final File sourceFile = sourceFiles[i1];

            final FileInputStream fileInputStream = new FileInputStream(sourceFile);
            final FileOutputStream fileOutputStream = new FileOutputStream(new File(databaseDir, sourceFile.getName()));
            try {
                int bytesRead;
                final byte[] buffer = new byte[1024];
                while ((bytesRead = fileInputStream.read(buffer, 0, buffer.length)) != -1) {
                    fileOutputStream.write(buffer, 0, bytesRead);
                }
            } finally {
                try {
                    fileInputStream.close();
                } catch (IOException e) {
                    // nothing we could do about it
                }
                try {
                    fileOutputStream.close();
                } catch (IOException e) {
                    // nothing we could do about it
                }
            }
        }
    }

    /**
     * This method checks whether the given two instances of DocumentIndex are identical.
     *
     * @param original  one instance of the comparison
     * @param retrieved the second instance of the comparison
     *
     * @throws junit.framework.AssertionFailedError
     *          in case the comparison does not match
     */
    protected static void assertDocumentIndicesEqual(final DocumentIndex original, final DocumentIndex retrieved) {
        assertEquals("word source id does not match", original.getWordSourceId(), retrieved.getWordSourceId());
        final Map words1 = original.getWords();
        final Map words2 = retrieved.getWords();
        assertEquals("Word list length does not match", words1.size(), words2.size());
        final Set set = words1.keySet();
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
            final CollationKey collationKey = (CollationKey) iterator.next();
            assertEquals("Indices list for word [" + collationKey.getSourceString() + "] does not match"
                , words1.get(collationKey), words2.get(collationKey));
        }
    }

    /**
     * This method checks whether the given two instances of CollectionIndex are identical.
     *
     * @param original  one instance of the comparison
     * @param retrieved the second instance of the comparison
     *
     * @throws junit.framework.AssertionFailedError
     *          in case the comparison does not match
     */
    protected static void assertCollectionIndicesEqual(final CollectionIndex original,
                                                       final CollectionIndex retrieved) {
        assertEquals("collection id does not match", original.getId(), retrieved.getId());
        final Set allDocumentIds1 = original.getAllDocumentIds();
        final Set allDocumentIds2 = retrieved.getAllDocumentIds();
        assertEquals("document count does not match", allDocumentIds1.size(), allDocumentIds2.size());
        for (Iterator iterator = allDocumentIds1.iterator(); iterator.hasNext();) {
            final Object o = iterator.next();
            assertTrue("document id [" + o + "] not contained in copy", allDocumentIds2.contains(o));
        }
        final Map words1 = original.getWords();
        final Map words2 = retrieved.getWords();
        assertEquals("Word list length does not match", words1.size(), words2.size());
        final Set set = words1.keySet();
        for (Iterator iterator = set.iterator(); iterator.hasNext();) {
            final CollationKey collationKey = (CollationKey) iterator.next();
            assertEquals("Indices list for word [" + collationKey.getSourceString() + "] does not match"
                , words1.get(collationKey), words2.get(collationKey));
        }
    }

    /**
     * This method checks whether the default XML persistence directory contains the same data as the directory with the
     * given path.
     *
     * @param path the path to the expected XML persistence state, for comparing with the actual current XML persistence
     *             directory content
     *
     * @throws junit.framework.AssertionFailedError
     *                   in case the comparison does not match
     * @throws Exception in case some unexpected Exception occurs
     */
    protected static void assertXMLDatabaseValidAndIdenticalTo(final String path) throws Exception {
        final File expectedState = new File("test_files/expectedDatabaseStates", path);
        final File currentState = new File(DEFAULT_PERSISTENCE_DIR_PATH);
        final File[] expectedFiles = expectedState.listFiles();
        for (int i = 0; i < expectedFiles.length; i++) {
            final File expectedFile = expectedFiles[i];
            final File actualFile = new File(currentState, expectedFile.getName());
            assertTrue("File [" + actualFile.getAbsolutePath() + "] is missing", actualFile.exists());
            XMLUnit.setIgnoreWhitespace(true);
            final Document expectedDocument = XMLUnit.buildControlDocument(new InputSource(new FileInputStream(
                expectedFile)));
            final Document actualDocument = XMLUnit.buildTestDocument(new InputSource(new FileInputStream(
                actualFile)));

            final Diff diff = XMLUnit.compareXML(expectedDocument, actualDocument);
            assertTrue(diff.toString(), diff.identical());
        }
    }
}
