/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.db;

import com.topcoder.document.index.persistence.IndexPersistenceException;
import junit.framework.Assert;

import java.sql.Connection;
import java.util.List;


/**
 * This class contains the unit tests for {@link DatabaseUtils}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DatabaseUtilsTest extends BaseDocumentIndexerPersistenceTest {
    /**
     * This is a name used in test cases.
     */
    private static final String COLLECTION_ID = "collection_id";
    /**
     * These are query args used commonly in most test cases.
     */
    private static final Object[] QUERY_ARGS = new Object[]{COLLECTION_ID};
    /**
     * This is the persistence instance used in queries for connection creation etc.
     */
    private AbstractDBIndexPersistence persistence;

    /**
     * This method tests {@link DatabaseUtils#closeSilently(java.sql.Connection)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testCloseSilently() throws Exception {
        final Connection connection = persistence.getConnection();
        connection.close();

        // may throw exception as connection already closed
        DatabaseUtils.closeSilently(connection);
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#closeSilently(java.sql.Connection)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> argument given
     */
    public void testCloseSilentlyFailureNull() {
        try {
            DatabaseUtils.closeSilently(null);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link DatabaseUtils#doQuery(java.sql.Connection, AbstractDBIndexPersistence, String, Object[],
     * DatabaseUtils.DataType[])}, {@link DatabaseUtils#doDMLQuery(java.sql.Connection, AbstractDBIndexPersistence,
     * String, Object[])} and {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection, AbstractDBIndexPersistence,
     * String, Object[], DatabaseUtils.DataType)} and {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testQueries() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, persistence,
                "INSERT INTO DOCUMENT_COLLECTION (COLLECTION_ID) VALUES (?)",
                new Object[]{COLLECTION_ID});

            final Object result = DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COUNT(*) FROM DOCUMENT_COLLECTION",
                new Object[0], DatabaseUtils.INTEGER_TYPE);
            Assert.assertEquals(1, ((Integer) result).intValue());

            List list = DatabaseUtils.doQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.assertEquals(1, list.size());
            final Object o = list.get(0);
            Assert.assertEquals(Object[].class, o.getClass());
            final Object[] array = (Object[]) o;
            Assert.assertEquals(1, array.length);
            Assert.assertEquals(COLLECTION_ID, array[0]);

            list = DatabaseUtils.doSingleColumnQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION",
                new Object[0],
                DatabaseUtils.STRING_TYPE);
            Assert.assertEquals(1, list.size());
            Assert.assertEquals(COLLECTION_ID, list.get(0));
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureNullConnection() throws Exception {
        try {
            DatabaseUtils.doQuery(null, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> persistence given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureNullPersistence() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, null,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureNullQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, persistence, null, QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: an empty String query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureEmptyQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, persistence, "  ", QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: invalid query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureInvalidQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, persistence, "SELECT bllubb", QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> queryArgs given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureNullQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", null,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: column type array does not match result set column count
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureIllegalColumnTypes() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE,
                    DatabaseUtils.INTEGER_TYPE,
                    DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: column type array contains a <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureNullInColumnTypes() throws Exception {
        final Connection connection = persistence.getConnection();
        DatabaseUtils.doDMLQuery(connection, persistence, "INSERT INTO DOCUMENT_COLLECTION (COLLECTION_ID) VALUES (?)",
            new Object[]{COLLECTION_ID});
        try {
            DatabaseUtils.doQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS,
                new DatabaseUtils.DataType[]{null});
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: column type array is <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureNullColumnTypes() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS,
                null);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: invalid queryArgs given (count does not match query)
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureInvalidQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                new Object[0],
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType[])}.
     * <p/>
     * <b>Failure reason</b>: closed connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoQueryFailureClosedConnection() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            connection.close();
            DatabaseUtils.doQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS,
                new DatabaseUtils.DataType[]{DatabaseUtils.STRING_TYPE});
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureNullConnection() throws Exception {
        try {
            DatabaseUtils.doSingleValueQuery(null, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS, DatabaseUtils.STRING_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> persistence given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureNullPersistence() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleValueQuery(connection, null,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureNullQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleValueQuery(connection, persistence, null, QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: an empty String query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureEmptyQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleValueQuery(connection, persistence, "   ", QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: invalid query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureInvalidQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleValueQuery(connection, persistence, "SELECT bllubb", QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> queryArgs given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureNullQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", null,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> columnType given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureNullColumnType() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS, null);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: invalid queryArgs given (count does not match query)
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureInvalidQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                new Object[]{COLLECTION_ID, "a"},
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: closed connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureClosedConnection() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            connection.close();
            DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS, DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleValueQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: the given query does return more that one column
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleValueQueryFailureIllegalQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        DatabaseUtils.doDMLQuery(connection, persistence, "INSERT INTO DOCUMENT (DOCUMENT_ID,LOCALE) VALUES (?,?)",
            new Object[]{COLLECTION_ID, "b"});
        try {
            DatabaseUtils.doSingleValueQuery(connection, persistence,
                "SELECT DOCUMENT_ID,LOCALE from DOCUMENT WHERE DOCUMENT_ID=?",
                QUERY_ARGS, DatabaseUtils.STRING_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureNullConnection() throws Exception {
        try {
            DatabaseUtils.doSingleColumnQuery(null, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS, DatabaseUtils.STRING_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> persistence given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureNullPersistence() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleColumnQuery(connection, null,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureNullQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleColumnQuery(connection, persistence, null, QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: an empty String query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureEmptyQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleColumnQuery(connection, persistence, "  ", QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: invalid query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureInvalidQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleColumnQuery(connection, persistence, "SELECT bllubb", QUERY_ARGS,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> queryArgs given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureNullQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleColumnQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", null,
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> columnType given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureNullColumnType() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleColumnQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?", QUERY_ARGS, null);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: invalid queryArgs given (count does not match query)
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureInvalidQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doSingleColumnQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                new Object[]{COLLECTION_ID, "a"},
                DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: closed connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureClosedConnection() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            connection.close();
            DatabaseUtils.doSingleColumnQuery(connection, persistence,
                "SELECT COLLECTION_ID FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS, DatabaseUtils.INTEGER_TYPE);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doSingleColumnQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[], DatabaseUtils.DataType)}.
     * <p/>
     * <b>Failure reason</b>: the given query does return more that one column
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoSingleColumnQueryFailureIllegalQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        DatabaseUtils.doDMLQuery(connection, persistence, "INSERT INTO DOCUMENT (DOCUMENT_ID,LOCALE) VALUES (?,?)",
            new Object[]{COLLECTION_ID, "b"});
        try {
            DatabaseUtils.doSingleColumnQuery(connection, persistence,
                "SELECT DOCUMENT_ID,LOCALE from DOCUMENT WHERE DOCUMENT_ID=?",
                QUERY_ARGS, DatabaseUtils.STRING_TYPE);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureNullConnection() throws Exception {
        try {
            DatabaseUtils.doDMLQuery(null, persistence, "DELETE FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> persistence given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureNullPersistence() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, null, "DELETE FROM DOCUMENT_COLLECTION WHERE COLLECTION_ID=?",
                QUERY_ARGS);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureNullQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, persistence, null, QUERY_ARGS);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: an empty String query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureEmptyQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, persistence, "   ", QUERY_ARGS);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: invalid query given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureInvalidQuery() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, persistence, "DELETE bllubb", QUERY_ARGS);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> queryArgs given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureNullQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, persistence,
                "DELETE FROM DOCUMENT_COLLECTION WHERE DOCUMENT_ID=?", null);
            Assert.fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: invalid queryArgs given (count does not match query)
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureInvalidQueryArgs() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            DatabaseUtils.doDMLQuery(connection, persistence,
                "DELETE FROM DOCUMENT_COLLECTION WHERE DOCUMENT_ID=?",
                new Object[0]);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DatabaseUtils#doDMLQuery(java.sql.Connection,
     * AbstractDBIndexPersistence, String, Object[])}.
     * <p/>
     * <b>Failure reason</b>: closed connection given
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDoDMLQueryFailureClosedConnection() throws Exception {
        final Connection connection = persistence.getConnection();
        try {
            connection.close();
            DatabaseUtils.doDMLQuery(connection, persistence,
                "DELETE FROM DOCUMENT_COLLECTION WHERE DOCUMENT_ID=?",
                QUERY_ARGS);
            Assert.fail("should throw");
        } catch (IndexPersistenceException expected) {
            //expected
        } finally {
            DatabaseUtils.closeSilently(connection);
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        persistence = new SimpleAccessDBIndexPersistence(SIMPLE_DB_CONFIGURATION_NAMESPACE);
    }

}
