/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.document.index.persistence.impl.xml;

import com.topcoder.document.index.DocumentIndex;
import com.topcoder.document.index.persistence.AbstractTestForIndexPersistence;
import com.topcoder.document.index.persistence.IndexPersistence;
import com.topcoder.document.index.persistence.impl.PersistenceConfigurationException;
import com.topcoder.document.index.wordsource.WordSourceId;

import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Map;
import java.util.Set;


/**
 * This class contains all unit tests for {@link XmlIndexPersistence}.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlIndexPersistenceTest extends AbstractTestForIndexPersistence {
    /**
     * This is a file size limit value used in test cases.
     */
    private static final int FILE_SIZE_LIMIT = 200;
    /**
     * This is a file size threshold value used in test cases.
     */
    private static final int FILE_SIZE_THRESHOLD = 100;

    /**
     * This method tests {@link XmlIndexPersistence#XmlIndexPersistence(String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtor() throws Exception {
        new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the given namespace argument is  <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNull() throws Exception {
        try {
            new XmlIndexPersistence(null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the given namespace argument is an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailEmpty() throws Exception {
        try {
            new XmlIndexPersistence("    ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: no configuration has been loaded
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoConfig() throws Exception {
        clearConfig();
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        } finally {
            loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeLimit was not parseable
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailUnparseableFileSizeLimit() throws Exception {
        loadXmlConfiguration("xml_invalidFileSizeLimit.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeLimit was a negative number
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNegativeFileSizeLimit() throws Exception {
        loadXmlConfiguration("xml_negativeFileSizeLimit.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeLimit was an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailEmptyFileSizeLimit() throws Exception {
        loadXmlConfiguration("xml_emptyFileSizeLimit.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeLimit was missing
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoFileSizeLimit() throws Exception {
        loadXmlConfiguration("xml_noFileSizeLimit.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeThreshold was not parseable
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailUnparseableFileSizeThreshold() throws Exception {
        loadXmlConfiguration("xml_invalidFileSizeThreshold.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeThreshold was a negative number
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNegativeFileSizeThreshold() throws Exception {
        loadXmlConfiguration("xml_negativeFileSizeThreshold.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeThreshold was an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailEmptyFileSizeThreshold() throws Exception {
        loadXmlConfiguration("xml_emptyFileSizeThreshold.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for fileSizeThreshold was missing
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoFileSizeThreshold() throws Exception {
        loadXmlConfiguration("xml_noFileSizeThreshold.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for persistencePath denoted a nonexistent path
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNotExistPersistencePath() throws Exception {
        loadXmlConfiguration("xml_notexistPersistencePath.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        } finally {
            loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for persistencePath denoted a file
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailIsFilePersistencePath() throws Exception {
        loadXmlConfiguration("xml_isFilePersistencePath.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        } finally {
            loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for persistencePath denoted a directory that did not contain a
     * valid document_usage.xml
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoDocumentUsagePersistencePath() throws Exception {
        loadXmlConfiguration("xml_noDocumentUsagePersistencePath.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        } finally {
            loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for persistencePath was an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailEmptyPersistencePath() throws Exception {
        loadXmlConfiguration("xml_emptyPersistencePath.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        } finally {
            loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(String)}.
     * <p/>
     * <b>Failure reason</b>: the config manager property for persistencePath was missing
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testOneArgCtorFailNoPersistencePath() throws Exception {
        loadXmlConfiguration("xml_noPersistencePath.xml");
        try {
            new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
            fail("should throw");
        } catch (PersistenceConfigurationException expected) {
            //expected
        } finally {
            loadFullConfiguration(DEFAULT_PERSISTENCE_CONFIGURATION);
        }
    }

    /**
     * This method tests {@link XmlIndexPersistence#XmlIndexPersistence(int, int, String)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtor() throws Exception {
        new XmlIndexPersistence(FILE_SIZE_LIMIT, FILE_SIZE_THRESHOLD, DEFAULT_PERSISTENCE_DIR_PATH);
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(int, int,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistencePath is <tt>null</tt>
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtorFailNull() throws Exception {
        try {
            new XmlIndexPersistence(FILE_SIZE_LIMIT, FILE_SIZE_THRESHOLD, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(int, int,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistencePath is an empty String
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtorFailEmpty() throws Exception {
        try {
            new XmlIndexPersistence(FILE_SIZE_LIMIT, FILE_SIZE_THRESHOLD, "  ");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(int, int,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistencePath is an nonexistent directory
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtorFailNotExist() throws Exception {
        try {
            new XmlIndexPersistence(FILE_SIZE_LIMIT, FILE_SIZE_THRESHOLD, "xxx");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(int, int,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistencePath is a file
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtorFailIsFile() throws Exception {
        try {
            new XmlIndexPersistence(FILE_SIZE_LIMIT, FILE_SIZE_THRESHOLD,
                "test_files/configurations/defaultConfig.xml");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(int, int,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given persistencePath does not contain a document_usage.xml
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtorFailNoDocumentUsage() throws Exception {
        try {
            new XmlIndexPersistence(FILE_SIZE_LIMIT, FILE_SIZE_THRESHOLD, "test_files/db1");
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(int, int,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given fileSizeLimit is less than 1
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtorFailZeroFileSize() throws Exception {
        try {
            new XmlIndexPersistence(0, FILE_SIZE_THRESHOLD, DEFAULT_PERSISTENCE_DIR_PATH);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link XmlIndexPersistence#XmlIndexPersistence(int, int,
     * String)}.
     * <p/>
     * <b>Failure reason</b>: the given fileSizeThreshold is less than 0
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testThreeArgCtorFailNegativeFileSizeThreshold() throws Exception {
        try {
            new XmlIndexPersistence(1, -1, DEFAULT_PERSISTENCE_DIR_PATH);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests {@link IndexPersistence#addDocumentIndex(DocumentIndex)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndex() throws Exception {
        super.testAddDocumentIndex();
        assertXMLDatabaseValidAndIdenticalTo("singleDocument");
    }

    /**
     * This method tests {@link XmlIndexPersistence#addDocumentIndex( DocumentIndex)} for correctness.
     * <p/>
     * <b>Special Scenario:</b> a somewhat larger index is persisted
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddLargeDocumentIndex() throws Exception {
        super.testAddLargeDocumentIndex();
        assertXMLDatabaseValidAndIdenticalTo("largeDocument");
    }

    /**
     * This method tests {@link XmlIndexPersistence#addDocumentIndex( DocumentIndex)} for correctness.
     * <p/>
     * <b>Special Scenario:</b> a somewhat larger index is persisted and the threshold is so large that multiple files
     * will be created
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddLargeDocumentIndexMultipleFiles() throws Exception {
        final Locale locale = Locale.getDefault();
        final WordSourceId sourceId = new WordSourceId("asdasd", "dsfsdgfds", new String[]{"a", "b"}, locale);
        final Map words = new HashMap();
        final Set sourceIds = new HashSet();
        sourceIds.add(sourceId);
        final Collator collator = Collator.getInstance(locale);
        final ArrayList indices = new ArrayList();
        indices.add(new Integer(3));
        indices.add(new Integer(7));
        for (int i = 0; i < 10000; i++) {
            final CollationKey key = collator.getCollationKey("WORD" + i);
            words.put(key, indices);
        }
        final DocumentIndex index = new DocumentIndex(sourceId, words);

        final XmlIndexPersistence persistence1 = new XmlIndexPersistence(1, 800, DEFAULT_PERSISTENCE_DIR_PATH);
        persistence1.addDocumentIndex(index);
        assertDocumentIndicesEqual(index, persistence1.getDocumentIndex(sourceId));
        assertXMLDatabaseValidAndIdenticalTo("largeDocumentMultipleFiles");
    }

    /**
     * This method tests {@link XmlIndexPersistence#addDocumentIndex( DocumentIndex)} for correctness.
     * <p/>
     * <b>Special Scenario:</b> no words in index
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndexNoWords() throws Exception {
        super.testAddDocumentIndexNoWords();
        assertXMLDatabaseValidAndIdenticalTo("noWordsDocument");
    }

    /**
     * This method tests {@link XmlIndexPersistence#addDocumentIndex( DocumentIndex)} for correctness.
     * <p/>
     * <b>Special Scenario:</b> delimiters are special chars that might need to be escaped
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddDocumentIndexSpecialDelimiters() throws Exception {
        super.testAddDocumentIndexSpecialDelimiters();
        assertXMLDatabaseValidAndIdenticalTo("specialDelimsDocument");
    }

    /**
     * This method tests {@link XmlIndexPersistence#addCollectionIndex(com.topcoder.document.index.CollectionIndex)} for
     * correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndex() throws Exception {
        super.testAddCollectionIndex();
        assertXMLDatabaseValidAndIdenticalTo("collection1Document");
    }

    /**
     * This method tests {@link XmlIndexPersistence#addCollectionIndex(com.topcoder.document.index.CollectionIndex)} for
     * correctness.
     * <p/>
     * <b>Special Scenario</b>: the given index contains multiple documents
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testAddCollectionIndexWithMultipleDocuments() throws Exception {
        super.testAddCollectionIndexWithMultipleDocuments();
        assertXMLDatabaseValidAndIdenticalTo("collectionMultipleDocument");
    }

    /**
     * This method tests {@link XmlIndexPersistence#updateCollectionIndex(com.topcoder.document.index.CollectionIndex)}
     * for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testUpdateCollectionIndex() throws Exception {
        super.testUpdateCollectionIndex();
        assertXMLDatabaseValidAndIdenticalTo("collectionMultipleDocument");
    }

    /**
     * This method tests {@link XmlIndexPersistence#increaseDocumentUseCount( WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testIncreaseDocumentUseCount() throws Exception {
        super.testIncreaseDocumentUseCount();
        assertXMLDatabaseValidAndIdenticalTo("usecountIncreased");
    }

    /**
     * This method tests {@link XmlIndexPersistence#decreaseDocumentUseCount( WordSourceId)} for correctness.
     *
     * @throws Exception in case an unexpected Exception occurs
     */
    public void testDecreaseDocumentUseCount() throws Exception {
        super.testDecreaseDocumentUseCount();
        assertXMLDatabaseValidAndIdenticalTo("usecountDecreased");
    }

    /**
     * This method creates the instance to be tested in the super class test cases.
     *
     * @return the instance to be tested
     *
     * @throws Exception in case some unexpected exception occurs
     */
    protected IndexPersistence createPersistenceInstance() throws Exception {
        return new XmlIndexPersistence(XML_CONFIGURATION_NAMESPACE);
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        clearXMLPersistence();
        super.setUp();
    }

    /**
     * This method does the cleanup after test run.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        clearXMLPersistence();
    }
}
